





'''
	import crowns.climate.scan as crowns_climate_scan
	crowns_ganglia = crowns_climate_scan.start ("crowns ganglia")
'''




import pathlib
from os.path import dirname, join, normpath
import sys
import copy
import json
import os.path
import inspect
import os

import ships.paths.files.scan.JSON as scan_JSON_path

'''
	if current directory is /1/1/1/1/1/
	
	looks in each directory for a crowns.JSON file
	
	possibilities:
		crowns.yaml
'''

name = "crowns.JSON"

def start ():
	#search_directory = os.path.abspath ((inspect.stack()[2])[1])
	#print ("search_directory:", search_directory)

	search_directory = os.getcwd ()
	
	searched = []
	while (search_directory != "/"):
		search_file_path = normpath (join (search_directory, name))
		print ("searching:", search_file_path)
		
		crowns_JSON_exists = os.path.isfile (search_file_path) 
		if (crowns_JSON_exists):
			print ("crowns.JSON found:", search_file_path)
		
			return {
				"configuration": scan_JSON_path.start (search_file_path),
				"file_path": search_file_path,
				"directory_path": search_directory
			}
		
		searched.append (search_file_path)
	
		search_directory = os.path.dirname (search_directory)
		
	 
	
	print (f"""
		
A "crowns.JSON" file was not found.
		
These paths were searched: { json.dumps (searched, indent = 4) }
		
	""")
	
	return False;






'''
	import crowns.ganglia.list as crowns_ganglia_list
	crowns_ganglia_list.start ()
'''


'''
	[{
		"name": "",
		"definitions": []
	}]
'''

import crowns.climate as crowns_climate
from pathlib import Path

import os
from rich import print_json

def start (
	find_name = None,
	return_multiple = False
):	
	treasuries = crowns_climate.find ("treasuries")	
	treasuries_path = treasuries ['path']
	
	issues = []
	
	proceeds = []
	
	directory_names = []
	for address in Path (treasuries_path).iterdir ():
		address_name = os.path.relpath (address, treasuries_path)
		
		print ("address_name", address_name)
		
		proceed = {
			"name": address_name,
			"definitions": []
		}
		
		if address.is_dir ():
			for treasury_path in Path (address).iterdir ():
				if treasury_path.is_dir ():
					treasury_name = os.path.relpath (treasury_path, address)
					treasury_name_split = treasury_name.split ('.')
					
					last_index_of_treasury = len (treasury_name_split) - 1;
					treasury_definition_number = int (treasury_name_split [ last_index_of_treasury ])
					
					assert (type (treasury_definition_number) == int), treasury_path
					proceed ["definitions"].append (treasury_name)
					
					print (find_name, treasury_name)
					
					
					
				else:
					issues.append ({
						"exception": "This module address is not a directory.",
						"path": str (treasury_path)
					})
				
				
		else:
			issues.append ({
				"exception": "This address is not a directory.",
				"path": address
			})
		
			#raise Exception (f'found a path that is not a directory: \n\n\t{ name }\n')
		
		if (find_name == address_name):
			return proceed
		
		proceeds.append (proceed)
	
		'''
		if trail.is_file ():
			print(f"{trail.name}:\n{trail.read_text()}\n")
		'''
	
	if (len (issues) >= 1):
		print_json (data = issues)
		raise Exception (f"Issues were found with the crowns directory.")
	
	if (type (find_name) == str):
		raise Exception (f"The name given was not found.")
	
	if (return_multiple):
		return {
			"proceeds": proceeds,
			"issues": issues
		}
	
	return proceeds




import crowns.mints.names as mints_names
import crowns.modules.moves.save as save

def clique ():
	import click
	@click.group ("mints")
	def group ():
		pass


	'''
		crowns mints names
	'''
	import click
	@group.command ("names")
	def names ():
		the_names = mints_names.start ()
		sorted_mints_names = sorted (the_names)
	
		for name in sorted_mints_names:
			print (name)
	
		return;

	'''
		crowns mints send-to-trends --name "mint-1"
	'''
	import click
	@group.command ("send-to-trends")
	@click.option ('--name', required = True)
	def search (name):
		#print ("name:", name)
		#print ('not yet implemented')
		
		save.save (
			name = name
		)
	
		return;

	return group




#







import pathlib
import inspect
import os
from os.path import dirname, join, normpath

from crowns.clique import clique
import crowns.config.scan as config_scan
import crowns.climate as crowns_climate

print ("crowns @:", pathlib.Path (__file__).parent.resolve ())

configured = False

import rich

def is_configured ():
	return configured

def start ():
	crowns_config = config_scan.start ()
	if (crowns_config == False): 
		return;

	'''
	rich.print_json (data = {
		"crowns_config": crowns_config
	})
	'''
	
	'''
	crowns_climate.change ("mongo", {
		"directory": ""
	})
	'''
	
	'''
		get the absolute paths
	'''
	'''
	crowns_config ["configuration"] ["treasuries"] ["path"] = (
		normpath (join (
			crowns_config ["directory_path"], 
			crowns_config ["configuration"] ["treasuries"] ["path"]
		))
	)
	'''
	
	
	'''
		paths:
			trends
				mongo_data_1
	
	
		mongo:
			safety
				passes
				zips
				zips.files
	'''
	trends_path = normpath (join (
		crowns_config ["directory_path"], 
		crowns_config ["configuration"] ["trends"] ["path"]
	))
	edited_config = {
		"mints": {
			"path": normpath (join (
				crowns_config ["directory_path"], 
				crowns_config ["configuration"] ["mints"] ["path"]
			))
		},
		"trends": {
			"path": trends_path,
			
			"nodes": [{
				"host": "localhost",
				"port": "27017",
				"data path": normpath (join (
					trends_path, 
					"mongo_data_1"
				))
			}]
		},
		"CWD": crowns_config ["directory_path"]
	}
	
	'''
	config_template = {
		
	}
	'''
	
	rich.print_json (data = {
		"edited_config": edited_config
	})

	
	crowns_climate.change ("edited_config", edited_config)
	

	#print ('crowns configuration', crowns_config.configuration)

	'''
		Add the changed version of the basal config
		to the climate.
	'''
	'''
	config = crowns_config ["configuration"];
	for field in config: 
		crowns_climate.change (field, config [field])
	'''
	
	configured = True
	
	print ()





'''
	#
	#	change
	#
	import crowns.climate as crowns_climate
	crowns_climate.change ("treasuries", {
		"path": treasuries_path
	})
	
	crowns_climate.change ("mongo", {
		"directory": ""
	})
'''


'''
	#
	#	find
	#
	import crowns.climate as crowns_climate
	mongo_climate = crowns_climate.find ("mongo")
'''

'''

'''

import copy

climate = {
	"mongo": {
		"directory": "",
		"connection": {
			"host": "localhost",
			"port": "27017"
		},
		"connection": "mongodb://localhost:27017/",
		"DB_name": 'safety',
		"passes": {
			"collection": "passes",
			"GridFS_zips": 'zips',
			"GridFS_zips_files": 'zips.files'
		}
	}	
}


'''
	import climate
	mongo = climate.connect ()
'''
from pymongo import MongoClient
import gridfs
def connect ():
	edited_config = find ("edited_config")
	
	nodes = edited_config ["trends"] ["nodes"]
	node_1 = nodes [0]
	
	host = node_1 ["host"]
	port = node_1 ["port"]

	mongo = MongoClient (f'mongodb://{ host }:{ port }/')
	
	return mongo;
	
def link ():
	return connect () ["safety"] ["passes"]


'''
	import climate
	GridFS = climate.link_FS ()
'''
def link_FS ():
	the_connection = connect ();
	DB = the_connection ["safety"]
	
	return gridfs.GridFS (DB)

def change (field, plant):
	#global CLIMATE;
	climate [ field ] = plant


def find (field):
	#print ("climate:", climate)

	return copy.deepcopy (climate) [ field ]





import crowns.climate as crowns_climate
import law_dictionary
import subprocess
def the_process (params):
	law_dictionary.check (	
		allow_extra_fields = True,
		laws = {
			"DB_directory": {
				"required": True,
				"type": str
			},
			"port": {
				"required": True,
				"type": str
			},
			"CWD": {
				"required": True,
				"type": str
			}
		},
		dictionary = params
	)

	port = params ["port"]
	DB_directory = params ["DB_directory"]
	CWD = params ["CWD"]

	subprocess.run (
		f"mongod --dbpath '{ DB_directory }' --port '{ port }'", 
		shell = True, 
		check = True,
		cwd = CWD
	)

	return;

from multiprocessing import Process
import rich
def turn_on ():
	mongo_climate = crowns_climate.find ("mongo")
	edited_config = crowns_climate.find ("edited_config")

	rich.print_json (data = {
		"mongo": mongo_climate,
		"edited_config": edited_config
	})
	

	'''
	return;
	mongo_DB_directory = move ["mongo directory"]
	mongo_port = move ["mongo port"]
	'''

	nodes = edited_config ["trends"] ["nodes"]
	for node in nodes:
		mongo = Process (
			target = the_process,
			args = (),
			kwargs = {
				"params": {
					"CWD": edited_config ["CWD"], 
					"DB_directory": node ["data path"],
					"port": str (node ["port"])
				}
			}
		)
		mongo.start ()


	





import crowns.modules.moves.trends_to_mint as trends_to_mint

import crowns.modules.moves.trends.find as find_trend
import crowns.modules.moves.trends.forget as forget_trend
import crowns.modules.moves.trends.on as on
import crowns.modules.moves.trends.show as show_trends

import click

def add ():
	
	@click.group ("trends")
	def group ():
		pass


	@group.command ("turn-on")
	def turn_on ():
		on.turn_on ()
	
	@group.command ("turn-off")
	def turn_off ():
		print ('not yet implemented')

	@group.command ("find")
	@click.option ('--name', required = True)
	def search (name):
		find_trend.find (name = name)
	
	@group.command ("list")
	def list_ ():
		show_trends.show ()
	
	@group.command ("show")
	def show ():
		show_trends.show ()
	
	@group.command ("forget")
	@click.option ('--id', required = True)
	@click.option ('--delete-if-file-gone', is_flag = True, default = False)
	def forget (id, delete_if_file_gone):		
		forget_trend.forget (
			id = id,
			delete_if_file_gone = delete_if_file_gone
		)
		
		

	'''
		crowns trends to-minds --name folder
	'''
	@group.command ("to-minds")
	@click.option ('--name', required = True)
	@click.option ('--id', required = False, default = '')
	def retrieve_command (name, id):
		trends_to_mint.start (
			name = name,
			id = id
		)

	return group




#











#from .group import clique as clique_group
from crowns.mints import clique as mints_group
from crowns.treasuries import clique as treasuries_group

import crowns.modules.moves.save as save





import crowns

import click

import crowns.clique.group.trends as group_trends

def clique ():
	'''
		This configures the crowns module.
	'''
	crowns.start ()

	@click.group ()
	def group ():
		pass
	
	@group.command ("help")
	def help ():
		import pathlib
		from os.path import dirname, join, normpath
		this_directory = pathlib.Path (__file__).parent.resolve ()
		this_module = str (normpath (join (this_directory, "..")))

		import somatic
		somatic.start ({
			"directory": this_module,
			"extension": ".s.HTML",
			"relative path": this_module
		})
		
		import time
		while True:
			time.sleep (1)

	
	
		

	


	group.add_command (mints_group.clique ())
	#group.add_command (treasuries_group.clique ())
	
	group.add_command (group_trends.add ())
	
	group ()




#






import click

def clique ():
	
	@click.group ("treasuries")
	def group ():
		pass

	'''
		crowns treasuries names
	'''
	@group.command ("names")
	def names ():
		import crowns.treasuries.names as treasuries_names
		names = treasuries_names.start ()
	
		for name in names:
			print (name)
	
	
	'''
		crowns treasuries definitions --name "hacienda"
	'''
	@group.command ("definitions")
	@click.option ('--name', required = True)
	def names (name):
		import crowns.treasuries.names as treasuries_names
		found_name = treasuries_names.start (
			find_name = name
		)
	
		print ('found:', found_name)
	
	

	@group.command ("enumerate")
	#@click.option ('--example-option', required = True)
	def search ():
		#print ("example_option:", example_option)
	
		return;

	return group




#









def add_paths_to_system (paths):
	import pathlib
	from os.path import dirname, join, normpath
	import sys
	
	this_folder = pathlib.Path (__file__).parent.resolve ()	
	for path in paths:
		sys.path.insert (0, normpath (join (this_folder, path)))

add_paths_to_system ([
	'../../../structures_pip_dependencies'
])



import pathlib
from os.path import dirname, join, normpath
this_folder = pathlib.Path (__file__).parent.resolve ()
modules = normpath (join (this_folder, "../modules"))

monitors = str (normpath (join (this_folder, "monitors")))

import sys
if (len (sys.argv) >= 2):
	glob_string = monitors + '/' + sys.argv [1]
else:
	glob_string = monitors + '/**/monitor_*.py'

import bracelet
scan = bracelet.start (
	glob_string = glob_string,
	simultaneous = True,
	module_paths = [
		normpath (join (modules, "structures")),
		normpath (join (modules, "structures_pip_dependencies"))
	],
	relative_path = monitors
)



#
#
#






'''
def change (field, plant):
	# check if the climate JSON exists
	climate_exists = os.path.isfile (climate_JSON_path) 
	if (climate_exists):
		current_JSON = scan_JSON_path.start (climate_JSON_path)
	else:
		print ('The climate.JSON was not found, therefore a new one is being built.')
		current_JSON = {}
		
		
	current_JSON [ field ] = plant;	

	FP = open (climate_JSON_path, "w")
	FP.write (json.dumps (current_JSON, indent = 4))
	FP.close ()

	print (current_JSON)
	return;

	#global climate;
	climate [ field ] = plant
'''
